﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.ApplicationModel.DataTransfer;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;
using Windows.System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;


namespace CallEditGeometryAppSample
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        private async void DoTest_Click(object sender, RoutedEventArgs e)
        {
            EditGeometry(true);
        }


        public async void EditGeometry(bool needResult)
        {
            bool isNew = true;

            ValueSet inputData = new ValueSet();
            if (isNew)
            {
            }
            else
            {
                FileOpenPicker openPicker = new FileOpenPicker();
                openPicker.ViewMode = PickerViewMode.List;
                openPicker.FileTypeFilter.Add(".sam");
                openPicker.FileTypeFilter.Add(".sdz");
                var gpxFile = await openPicker.PickSingleFileAsync();
                if (gpxFile == null)
                {
                    return;
                }

                var token = SharedStorageAccessManager.AddFile(gpxFile);
                inputData.Add("EditFileToken", token);
            }

            await EditGeometry(needResult, inputData);

        }

        private async System.Threading.Tasks.Task EditGeometry(bool needResult, ValueSet inputData)
        {
            var options = new LauncherOptions();
            options.TargetApplicationPackageFamilyName = "62301LeonLi.ActiveGeometry_496e9sjp76zt6";

            Uri uri = new Uri("ag.editgeometry:?");

            if (!needResult)
            {
                await Launcher.LaunchUriAsync(uri, options, inputData);
                ShowMessage.Text = "No Reuslt Success";
            }
            else
            {
                if (!IsIncludeProcess.IsChecked.Value)
                {
                    inputData.Add("IncludeProcess", "N");
                }

                LaunchUriResult resultData = await Launcher.LaunchUriForResultsAsync(uri, options, inputData);
                ShowMessage.Text = resultData.Status.ToString();
                if (resultData.Status == LaunchUriStatus.Success &&
                    resultData.Result != null &&
                    resultData.Result.ContainsKey("ReturnedState"))
                {
                    ValueSet theValues = resultData.Result;
                    var resultState = theValues["ReturnedState"] as string;
                    switch (resultState)
                    {
                        case "Success":
                            if (theValues.ContainsKey("ImgFileToken"))
                            {
                                var tokenObject = theValues["ImgFileToken"];
                                if (tokenObject != null)
                                {
                                    var token = tokenObject.ToString();
                                    var imgFile = await SharedStorageAccessManager.RedeemTokenForFileAsync(token);
                                    if (imgFile != null)
                                    {
                                        try
                                        {
                                            IRandomAccessStream imageStream = await imgFile.OpenReadAsync();
                                            BitmapImage bitmapImage = new BitmapImage();
                                            bitmapImage.SetSource(imageStream);
                                            ShowImage.Source = bitmapImage;
                                        }
                                        finally { }
                                    }
                                }
                            }

                            if (theValues.ContainsKey("DataFileToken"))
                            {
                                var tokenObject = theValues["DataFileToken"];
                                if (tokenObject != null)
                                {
                                    var token = tokenObject.ToString();
                                    var dataFile = await SharedStorageAccessManager.RedeemTokenForFileAsync(token);
                                    if (dataFile != null)
                                    {                                            
                                        var localFolder = Windows.Storage.ApplicationData.Current.LocalFolder;
                                        file_demonstration = await dataFile.CopyAsync(localFolder);
                                        DoEdit.Visibility = Visibility.Visible;
                                    }
                                }
                            }
                            ShowMessage.Text = "Success";
                            break;
                        case "Error":
                            var resultMessage = theValues["ReturnedMessage"] as string;
                            if (resultMessage == "Duplicate")
                            {
                                string mes = "Active Geometry has opened another windows. Please close it first.";
                                ShowMessage.Text = mes;
                            }

                            var isCloseByOther = theValues["ByOtherClosed"] as string;
                            if (isCloseByOther != "Y")
                            {
                                ShowMessage.Text += " ---   By Other Closed";
                            }
                            break;
                    }
                }
            }
        }

        StorageFile file_demonstration;

        private async void DoEdit_Click(object sender, RoutedEventArgs e)
        {
            if (file_demonstration == null)
                return;

            var token = SharedStorageAccessManager.AddFile(file_demonstration);
            ValueSet inputData = new ValueSet();
            inputData.Add("EditFileToken", token);
            await EditGeometry(true, inputData);
        }
    }
}